--[[
Interface: 1.4.4.0 1.4.4RC8
Author: GtX | Andy
Date: 21.06.2017
Version: 1.0
About: Field service trailer script. Seasons Ready. Repair mod coming soon.

Notes: Multiplayer Ready

Support: http://ls-modcompany.com

History:
V 1.0 @ 21.06.2017 - Project Started
V 1.1 @ 13/07/2017 - Improved functions added after testing.
]]

ServiceTrailer = {}

function ServiceTrailer.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
--		and SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function ServiceTrailer:load(savegame)
	self.setIsTurnedOn = ServiceTrailer.setIsTurnedOn
    self.getIsTurnedOnAllowed = ServiceTrailer.getIsTurnedOnAllowed
    self.getIsTurnedOn = ServiceTrailer.getIsTurnedOn
    self.onTurnedOn = SpecializationUtil.callSpecializationsFunction("onTurnedOn")
    self.onTurnedOff = SpecializationUtil.callSpecializationsFunction("onTurnedOff")	
	
	self.triggerCallback = VehicleSellingPoint.triggerCallback;
	self.sellAreaTriggerCallback = VehicleSellingPoint.sellAreaTriggerCallback;
	self.determineCurrentVehicle = VehicleSellingPoint.determineCurrentVehicle;
	self.onActivateObject = VehicleSellingPoint.onActivateObject;
	self.getIsActivatable = VehicleSellingPoint.getIsActivatable;
	self.drawActivate = VehicleSellingPoint.drawActivate;
	
	self.setServiceAnimation = ServiceTrailer.setServiceAnimation;
	self.playerServiceTrigger = SpecializationUtil.callSpecializationsFunction("playerServiceTrigger");
	self.playerFuelTrigger = SpecializationUtil.callSpecializationsFunction("playerFuelTrigger");
	self.isTankFilled = ServiceTrailer.isTankFilled;

	if self.isClient then
	
		self.serviceGroup = {};
	
		self.serviceGroup.sellTriggerId = Utils.indexToObject(self.components, getXMLString(self.xmlFile,"vehicle.serviceTrailer.service#playerTriggerIndex"));
		self.serviceGroup.sellAreaTriggerId = Utils.indexToObject(self.components, getXMLString(self.xmlFile,"vehicle.serviceTrailer.service#vehicleTriggerIndex"));
		self.activateText = g_i18n:getText("action_activateServiceMenu");
		
		addTrigger(self.serviceGroup.sellTriggerId, "triggerCallback", self);
		addTrigger(self.serviceGroup.sellAreaTriggerId, "sellAreaTriggerCallback", self);
		
		self.vehicleInRange = {}
		self.currentVehicle = nil
		self.currentVehicleId = 0    
			
		self.ownWorkshop = true;
		self.isEnabled = true
		self.objectActivated = false
		
		self.animService = {}	
	
		self.animService.playerTriggerId = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.serviceTrailer.serviceAnim#triggerIndex"));
		self.animService.animationName = getXMLString(self.xmlFile, "vehicle.serviceTrailer.serviceAnim#animationName");
	
		addTrigger(self.animService.playerTriggerId, "playerServiceTrigger", self );
		
		self.animService.player = false;
		self.serviceAnimActive = false
	
		
		self.operateFuel = {}
		
		self.operateFuel.playerTriggerId = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.serviceTrailer.operateFuel#triggerIndex"));	
		
	--	addTrigger(self.operateFuel.playerTriggerId, "playerFuelTrigger", self );
		
		self.operateFuel.isTurnedOn = false
		self.operateFuel.player = false;	
	
        self.operateFuel.rotationNodes = Utils.loadRotationNodes(self.xmlFile, {}, "vehicle.turnedOnRotationNodes.turnedOnRotationNode", "fuelPump", self.components)
		
        self.operateFuel.sampleMotorRun = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.serviceTrailer.runSound", nil, self.baseDirectory, self.components[1].node)
		
		self.operateFuel.runPS = {};
		Utils.loadParticleSystem(self.xmlFile, self.operateFuel.runPS, "vehicle.serviceTrailer.exhaust", self.components, false, nil, self.baseDirectory);
		
        
		local turnOnAnimation = getXMLString(self.xmlFile, "vehicle.serviceTrailer.operateFuel#animationName")
		
        if turnOnAnimation ~= nil then
            self.operateFuel.turnOnAnimation = {}
            self.operateFuel.turnOnAnimation.name = turnOnAnimation
            self.operateFuel.turnOnAnimation.turnOnSpeedScale = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.serviceTrailer.operateFuel#animationTurnOnSpeedScale"), 1)
            self.operateFuel.turnOnAnimation.turnOffSpeedScale = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.serviceTrailer.operateFuel#animationTurnOffSpeedScale"), -self.operateFuel.turnOnAnimation.turnOnSpeedScale)
        end;
    end;	
end;

function ServiceTrailer:readStream(streamId, connection)
	local state = streamReadBool(streamId);
	self:setServiceAnimation(state, true);
	local turnedOn = streamReadBool(streamId);
    self:setIsTurnedOn(turnedOn, true);
end;

function ServiceTrailer:writeStream(streamId, connection)
	local state = self.serviceAnimActive;
	streamWriteBool(streamId, state);
	streamWriteBool(streamId, self.operateFuel.isTurnedOn);		
end;

function ServiceTrailer:delete()
	if g_currentMission:getIsClient() then
		removeTrigger(self.serviceGroup.sellTriggerId)
		removeTrigger(self.serviceGroup.sellAreaTriggerId)
--		removeTrigger(self.operateFuel.playerTriggerId)
		removeTrigger(self.animService.playerTriggerId)
    end;	
	if self.isClient then
        SoundUtil.deleteSample(self.operateFuel.sampleMotorRun)
		Utils.deleteParticleSystem(self.operateFuel.runPS);
    end;
end;

function ServiceTrailer:playerServiceTrigger(triggerId, otherId, onEnter, onLeave, onStay)
	if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.animService.player = true;
	elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.animService.player = false;
	end;	
end;

function ServiceTrailer:playerFuelTrigger(triggerId, otherId, onEnter, onLeave, onStay)
	if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.operateFuel.player = true;
	elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.operateFuel.player = false;
	end;	
end;

function ServiceTrailer:setServiceAnimation(state, noEventSend)
	animationEvent.sendEvent(self, state, noEventSend);
	local animationTime = self:getAnimationTime(self.animService.animationName);
	
	if state and animationTime < 1 then
		self:playAnimation(self.animService.animationName, 1, animationTime, true);
	elseif state and animationTime >= 1 then
		self:playAnimation(self.animService.animationName, 1, 0, true);
	elseif animationTime > 0 then
		self:playAnimation(self.animService.animationName, -1, animationTime, true);
	end;
	self.serviceAnimActive = state
end;

function ServiceTrailer:update(dt)

	if self.isClient then	
		if self.animService.player == true then
			if InputBinding.hasEvent(InputBinding.service_activateService) then
				self:setServiceAnimation(not self.serviceAnimActive);
		end;
	
		local animationTimeS = self:getAnimationTime(self.animService.animationName)
			if animationTimeS > 0 then
				g_currentMission:addHelpButtonText(g_i18n:getText("action_deactivateService"), InputBinding.service_activateService);
					setTextAlignment(RenderText.ALIGN_CENTER); 				
					renderText( 0.5, 0.06, 0.02, g_i18n:getText("action_deactivateService"));		
			elseif animationTimeS < 1 then
				g_currentMission:addHelpButtonText(g_i18n:getText("action_activateService"), InputBinding.service_activateService);
					setTextAlignment(RenderText.ALIGN_CENTER); 				
					renderText( 0.5, 0.06, 0.02, g_i18n:getText("action_activateService"));
				end;
			end;
		end;
		
		if self.isClient then
			if self.operateFuel.player == true then
				if InputBinding.hasEvent(InputBinding.service_activateFuel) then
					local doNotStart = not self:isTankFilled()
					if doNotStart then
						g_currentMission:showBlinkingWarning(g_i18n:getText("warning_fuelTankEmpty"));
					else
						self:setIsTurnedOn(not self.operateFuel.isTurnedOn)
					end;
				end;
			
			if self.operateFuel.isTurnedOn then
				g_currentMission:addHelpButtonText(g_i18n:getText("action_stopFuelPump"), InputBinding.service_activateFuel);
				setTextAlignment(RenderText.ALIGN_CENTER); 				
				renderText( 0.5, 0.06, 0.02, g_i18n:getText("action_stopFuelPump"));
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("action_startFuelPump"), InputBinding.service_activateFuel);
				setTextAlignment(RenderText.ALIGN_CENTER); 				
				renderText( 0.5, 0.06, 0.02, g_i18n:getText("action_startFuelPump"));
			end;
		end;
	end;
		
	if self.isClient then
		if self:getIsTurnedOn() then
            SoundUtil.play3DSample(self.operateFuel.sampleMotorRun)
			Utils.setEmittingState(self.operateFuel.runPS, true);
        end;		
		Utils.updateRotationNodes(self, self.operateFuel.rotationNodes, dt, self:getIsTurnedOn())
    end;
	
	local emptyTank = not self:isTankFilled()
	if emptyTank then
		self:setIsTurnedOn(false, nil);
	end;
end;

function ServiceTrailer:getIsTurnedOn()
	return self.operateFuel.isTurnedOn
end;

function ServiceTrailer:setIsTurnedOn(isTurnedOn, noEventSend)
    if isTurnedOn ~= self.operateFuel.isTurnedOn then
        TurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
        self.operateFuel.isTurnedOn = isTurnedOn
        if self.operateFuel.isTurnedOn then
            self:onTurnedOn(noEventSend)
        else
            self:onTurnedOff(noEventSend)
        end;
    end;
end;

function ServiceTrailer:onTurnedOn(noEventSend)
	if self.isClient then
        if self.playAnimation ~= nil then
            if self.operateFuel.turnOnAnimation ~= nil then
                self:playAnimation(self.operateFuel.turnOnAnimation.name, self.operateFuel.turnOnAnimation.turnOnSpeedScale, self:getAnimationTime(self.operateFuel.turnOnAnimation.name), true)
            end;
        end;
    end; 
end;

function ServiceTrailer:onTurnedOff(noEventSend)
    if self.isClient then
        if self.playAnimation ~= nil then
            if self.operateFuel.turnOnAnimation ~= nil then
                self:playAnimation(self.operateFuel.turnOnAnimation.name, self.operateFuel.turnOnAnimation.turnOffSpeedScale, self:getAnimationTime(self.operateFuel.turnOnAnimation.name), true)
            end;
        end;
		
		SoundUtil.stop3DSample(self.operateFuel.sampleMotorRun)
		Utils.setEmittingState(self.operateFuel.runPS, false);
    end;
end;

function ServiceTrailer:isTankFilled()
	if self.getFillLevel ~= nil then
		return self:getFillLevel() > 0
	end;
	return false
end;

function ServiceTrailer:draw()
end;

function ServiceTrailer:updateTick(dt)
end;

function ServiceTrailer:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ServiceTrailer:keyEvent(unicode, sym, modifier, isDown)
end;

-- Operate Service Event --

animationEvent = {}
animationEvent_mt = Class(animationEvent, Event);
InitEventClass(animationEvent, "animationEvent");

function animationEvent:emptyNew()
    local self = Event:new(animationEvent_mt);
    return self;
end;

function animationEvent:new(object, state)
    local self = animationEvent:emptyNew();
    self.object = object;
	self.state = state;
    return self;
end;

function animationEvent:readStream(streamId, connection)
	self.object = readNetworkNodeObject(streamId);
	self.state = streamReadBool(streamId);	
    self:run(connection);
end;

function animationEvent:writeStream(streamId, connection)
	writeNetworkNodeObject(streamId, self.object);
	streamWriteBool(streamId, self.state);
end;

function animationEvent:run(connection)
    self.object:setServiceAnimation(self.state, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(animationEvent:new(self.object, self.state), nil, connection, self.object);
    end;
end;

function animationEvent.sendEvent(vehicle, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(animationEvent:new(vehicle, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(animationEvent:new(vehicle, state));
		end;
	end;
end;

-- Operate Fuel Event --

TurnedOnEvent = {};
TurnedOnEvent_mt = Class(TurnedOnEvent, Event);
InitEventClass(TurnedOnEvent, "TurnedOnEvent");


function TurnedOnEvent:emptyNew()
    local self = Event:new(SetTurnedOnEvent_mt);
    return self;
end;

function TurnedOnEvent:new(object, isTurnedOn)
    local self = TurnedOnEvent:emptyNew()
    self.object = object;
    self.isTurnedOn = isTurnedOn;
    return self;
end;

function TurnedOnEvent:readStream(streamId, connection)
    self.object = readNetworkNodeObject(streamId);
    self.isTurnedOn = streamReadBool(streamId);
    self:run(connection);
end;

function TurnedOnEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.object);
    streamWriteBool(streamId, self.isTurnedOn);
end;

function TurnedOnEvent:run(connection)
    if not connection:getIsServer() then
        g_server:broadcastEvent(self, false, connection, self.object);
    end;
    if self.object ~= nil then
        self.object:setIsTurnedOn(self.isTurnedOn, true);
    end;
end;

function TurnedOnEvent.sendEvent(vehicle, isTurnedOn, noEventSend)
    if isTurnedOn ~= vehicle.operateFuel.isTurnedOn then
        if noEventSend == nil or noEventSend == false then
            if g_server ~= nil then
                g_server:broadcastEvent(TurnedOnEvent:new(vehicle, isTurnedOn), nil, nil, vehicle);
            else
                g_client:getServerConnection():sendEvent(TurnedOnEvent:new(vehicle, isTurnedOn));
            end;
        end;
    end;
end;